"""Configuration for the speech prosody model."""
from typing import Any, Dict, Optional

from hume._common.config.job_config_base import JobConfigBase
from hume._common.model_type import ModelType


class ProsodyConfig(JobConfigBase["ProsodyConfig"]):
    """Configuration for the speech prosody model."""

    def __init__(
        self,
        *,
        identify_speakers: Optional[bool] = None,
    ):
        """Construct a `ProsodyConfig`.

        Args:
            identify_speakers (Optional[bool]): Whether to return identifiers for speakers over time. If true,
                unique identifiers will be assigned to spoken words to differentiate different speakers. If false,
                all speakers will be tagged with an "unknown" ID.
        """
        self.identify_speakers = identify_speakers

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.PROSODY

    def serialize(self) -> Dict[str, Any]:
        """Serialize `ProsodyConfig` to dictionary.

        Returns:
            Dict[str, Any]: Serialized `ProsodyConfig` object.
        """
        return {
            "identify_speakers": self.identify_speakers,
        }

    @classmethod
    def deserialize(cls, request_dict: Dict[str, Any]) -> "ProsodyConfig":
        """Deserialize `ProsodyConfig` from request JSON.

        Args:
            request_dict (Dict[str, Any]): Request JSON data.

        Returns:
            ProsodyConfig: Deserialized `ProsodyConfig` object.
        """
        return cls(identify_speakers=request_dict.get("identify_speakers"))



"""Streaming socket connection."""
import base64
import json
from pathlib import Path
from typing import Any, List, Union

try:
    from websockets.client import WebSocketClientProtocol
    HAS_WEBSOCKETS = True
except ModuleNotFoundError:
    HAS_WEBSOCKETS = False

from hume._common.config import JobConfigBase
from hume._common.hume_client_error import HumeClientError


class StreamSocket:
    """Streaming socket connection."""

    def __init__(
        self,
        protocol: "WebSocketClientProtocol",
        configs: List[JobConfigBase],
    ):
        """Construct a `StreamSocket`.

        Args:
            protocol (WebSocketClientProtocol): Protocol instance from websockets library.
            configs (List[JobConfigBase]): List of model configurations.

        Raises:
            HumeClientError: If there is an error processing media over the socket connection.
        """
        if not HAS_WEBSOCKETS:
            raise HumeClientError("websockets package required to use HumeStreamClient")

        self._configs = configs
        self._protocol = protocol

        self._serialized_configs = self._serialize_configs(configs)

    @classmethod
    def _serialize_configs(cls, configs: List[JobConfigBase]) -> Any:
        serialized = {}
        for config in configs:
            model_type = config.get_model_type()
            model_name = model_type.value
            serialized[model_name] = config.serialize()
        return serialized

    @classmethod
    def _file_to_bytes(cls, filepath: Path) -> bytes:
        with filepath.open('rb') as f:
            return base64.b64encode(f.read())

    def _get_predictions(self, response: str) -> Any:
        try:
            json_response = json.loads(response)
        except json.JSONDecodeError as exc:
            raise HumeClientError("Unexpected error when fetching streaming API predictions") from exc

        return json_response

    async def send_bytes_str(self, bytes_str: str) -> Any:
        """Send raw bytes string on the `StreamSocket`.

        Note: Must be ascii encoded bytes.

        Args:
            bytes_str (str): Raw bytes of media to send on socket connection converted to a string.

        Returns:
            Any: Predictions from the streaming API.
        """
        payload = {
            "data": bytes_str,
            "models": self._serialized_configs,
        }
        json_payload = json.dumps(payload)
        await self._protocol.send(json_payload)
        response = await self._protocol.recv()
        # Cast to str because websockets can send bytes, but we will always accept JSON strings
        response_str = str(response)
        return self._get_predictions(response_str)

    async def send_bytes(self, bytes_data: bytes) -> Any:
        """Send raw bytes on the `StreamSocket`.

        Args:
            bytes_data (bytes): Raw bytes of media to send on socket connection.

        Returns:
            Any: Predictions from the streaming API.
        """
        bytes_str = bytes_data.decode("ascii")
        return await self.send_bytes_str(bytes_str)

    async def send_file(self, filepath: Union[str, Path]) -> Any:
        """Send a file on the `StreamSocket`.

        Args:
            filepath (Path): Path to media file to send on socket connection.

        Returns:
            Any: Predictions from the streaming API.
        """
        bytes_data = self._file_to_bytes(Path(filepath))
        return await self.send_bytes(bytes_data)



"""Batch job result."""
import json
from pathlib import Path
from typing import Any, Dict, List, Optional, Union
from urllib.request import urlretrieve

from hume._batch.batch_job_status import BatchJobStatus
from hume._common.model_type import ModelType
from hume._common.config.job_config_base import JobConfigBase
from hume._common.config.config_utils import config_from_model_type
from hume._common.hume_client_error import HumeClientError


class BatchJobResult:
    """Batch job result."""

    def __init__(
        self,
        *,
        configs: Dict[ModelType, JobConfigBase],
        urls: List[str],
        status: BatchJobStatus,
        predictions_url: Optional[str] = None,
        artifacts_url: Optional[str] = None,
        errors_url: Optional[str] = None,
        error_message: Optional[str] = None,
    ):
        """Construct a BatchJobResult.

        Args:
            configs (Dict[ModelType, JobConfigBase]): Configurations for the `BatchJob`.
            urls (List[str]): URLs processed in the `BatchJob`.
            status (BatchJobStatus): Status of `BatchJob`.
            predictions_url (Optional[str]): URL to predictions file.
            artifacts_url (Optional[str]): URL to artifacts zip archive.
            errors_url (Optional[str]): URL to errors file.
            error_message (Optional[str]): Error message for request.
        """
        self.configs = configs
        self.urls = urls
        self.status = status
        self.predictions_url = predictions_url
        self.artifacts_url = artifacts_url
        self.errors_url = errors_url
        self.error_message = error_message

    def download_predictions(self, filepath: Optional[Union[str, Path]] = None) -> None:
        """Download `BatchJob` predictions file.

        Args:
            filepath (Optional[Union[str, Path]]): Filepath where predictions will be downloaded.
        """
        if self.predictions_url is None:
            raise HumeClientError("Could not download job predictions. No predictions found on job result.")
        urlretrieve(self.predictions_url, filepath)

    def download_artifacts(self, filepath: Optional[Union[str, Path]] = None) -> None:
        """Download `BatchJob` artifacts zip archive.

        Args:
            filepath (Optional[Union[str, Path]]): Filepath where artifacts zip archive will be downloaded.
        """
        if self.artifacts_url is None:
            raise HumeClientError("Could not download job artifacts. No artifacts found on job result.")
        urlretrieve(self.artifacts_url, filepath)

    def download_errors(self, filepath: Optional[Union[str, Path]] = None) -> None:
        """Download `BatchJob` errors file.

        Args:
            filepath (Optional[Union[str, Path]]): Filepath where errors will be downloaded.
        """
        if self.errors_url is None:
            raise HumeClientError("Could not download job errors. No errors found on job result.")
        urlretrieve(self.errors_url, filepath)

    def get_error_message(self) -> Optional[str]:
        """Get any available error messages on the job.

        Returns:
            Optional[str]: A string with the error message if there was an error, otherwise None.
        """
        return self.error_message

    @classmethod
    def from_response(cls, response: Any) -> "BatchJobResult":
        """Construct a `BatchJobResult` from a batch API job response.

        Args:
            response (Any): Batch API job response.

        Returns:
            BatchJobResult: A `BatchJobResult` based on a batch API job response.
        """
        try:
            request = response["request"]
            configs = {}
            for model_name, config_dict in request["models"].items():
                model_type = ModelType.from_str(model_name)
                config = config_from_model_type(model_type).deserialize(config_dict)
                configs[model_type] = config

            kwargs = {}
            if "completed" in response:
                completed_dict = response["completed"]
                kwargs["artifacts_url"] = completed_dict["artifacts_url"]
                kwargs["errors_url"] = completed_dict["errors_url"]
                kwargs["predictions_url"] = completed_dict["predictions_url"]

            if "failed" in response:
                failed_dict = response["failed"]
                if "message" in failed_dict:
                    kwargs["error_message"] = failed_dict["message"]

            return cls(
                configs=configs,
                urls=request["urls"],
                status=BatchJobStatus.from_str(response["status"]),
                **kwargs,
            )
        # pylint: disable=broad-except
        except Exception as exc:
            message = cls._get_invalid_response_message(response)
            raise HumeClientError(message) from exc

    @classmethod
    def _get_invalid_response_message(cls, response: Any) -> str:
        response_str = json.dumps(response)
        message = f"Could not parse response into BatchJobResult: {response_str}"

        # Check for invalid API key
        if "fault" in response and "faultstring" in response["fault"]:
            fault_string = response["fault"]["faultstring"]
            if fault_string == "Invalid ApiKey":
                message = "Client initialized with invalid API key"

        return message



"""Base class for Hume clients."""
from typing import Optional
from abc import ABC


class ClientBase(ABC):
    """Base class for Hume API clients."""

    _HTTP_BASE_URL = "https://api.hume.ai"
    _WS_BASE_URI = "wss://api.hume.ai"

    def __init__(
        self,
        api_key: str,
        _api_version: str = "v0",
        _api_http_base_url: Optional[str] = None,
        _api_ws_base_uri: Optional[str] = None,
    ):
        """Construct a new Hume API client.

        Args:
            api_key (str): Hume API key.
        """
        self._api_key = api_key
        self._api_version = _api_version
        self._api_http_base_url = self._HTTP_BASE_URL if _api_http_base_url is None else _api_http_base_url
        self._api_ws_base_uri = self._WS_BASE_URI if _api_ws_base_uri is None else _api_ws_base_uri



"""Streaming API client."""
import logging
from contextlib import asynccontextmanager
from typing import Any, AsyncIterator, List

from hume._common.api_type import ApiType
from hume._common.client_base import ClientBase
from hume._common.config import JobConfigBase
from hume._common.config.config_utils import config_from_model_type
from hume._common.hume_client_error import HumeClientError
from hume._common.model_type import ModelType
from hume._stream.stream_socket import StreamSocket

try:
    import websockets
    HAS_WEBSOCKETS = True
except ModuleNotFoundError:
    HAS_WEBSOCKETS = False

logger = logging.getLogger(__name__)


class HumeStreamClient(ClientBase):
    """Streaming API client.

    Example:
        ```python
        import asyncio

        from hume import HumeStreamClient, StreamSocket
        from hume.config import FaceConfig

        async def main():
            client = HumeStreamClient("<your-api-key>")
            configs = [FaceConfig(identify_faces=True)]
            async with client.connect(configs) as socket:
                socket: StreamSocket
                result = await socket.send_file("<your-image-filepath>")
                print(result)

        asyncio.run(main())
        ```
    """

    _DEFAULT_API_TIMEOUT = 10

    def __init__(self, *args: Any, **kwargs: Any):
        """Construct a HumeStreamClient.

        Args:
            api_key (str): Hume API key.
        """
        if not HAS_WEBSOCKETS:
            raise HumeClientError("websockets package required to use HumeStreamClient")

        super().__init__(*args, **kwargs)

    @asynccontextmanager
    async def connect(self, configs: List[JobConfigBase]) -> AsyncIterator:
        """Connect to the streaming API.

        Args:
            configs (List[JobConfigBase]): List of job configs.
        """
        uri = (f"{self._api_ws_base_uri}/{self._api_version}/{ApiType.STREAM.value}/multi"
               f"?apikey={self._api_key}")

        try:
            # pylint: disable=no-member
            async with websockets.connect(uri) as protocol:  # type: ignore[attr-defined]
                yield StreamSocket(protocol, configs)
        except websockets.exceptions.InvalidStatusCode as exc:
            message = "Client initialized with invalid API key"
            raise HumeClientError(message) from exc

    @asynccontextmanager
    async def _connect_to_models(self, configs_dict: Any) -> AsyncIterator:
        """Connect to the streaming API with a single models configuration dict.

        Args:
            configs_dict (Any): Models configurations dict. This should be a dict from model name
                to model configuration dict. An empty dict uses the default configuration.
        """
        configs = []
        for model_name, config_dict in configs_dict.items():
            model_type = ModelType.from_str(model_name)
            config = config_from_model_type(model_type).deserialize(config_dict)
            configs.append(config)

        async with self.connect(configs) as websocket:
            yield websocket



"""Configuration for the vocal burst model."""
from typing import Any, Dict

from hume._common.config.job_config_base import JobConfigBase
from hume._common.model_type import ModelType


class BurstConfig(JobConfigBase["BurstConfig"]):
    """Configuration for the vocal burst model."""

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.BURST

    def serialize(self) -> Dict[str, Any]:
        """Serialize `BurstConfig` to dictionary.

        Returns:
            Dict[str, Any]: Serialized `BurstConfig` object.
        """
        return {}

    @classmethod
    def deserialize(cls, request_dict: Dict[str, Any]) -> "BurstConfig":
        """Deserialize `BurstConfig` from request JSON.

        Args:
            request_dict (Dict[str, Any]): Request JSON data.

        Returns:
            BurstConfig: Deserialized `BurstConfig` object.
        """
        return cls()



"""Abstract base class for model configurations."""
from abc import ABC, abstractmethod
from typing import Any, Dict, Generic, TypeVar

from hume._common.model_type import ModelType

T = TypeVar("T")  # Type for subclasses of JobConfigBase


class JobConfigBase(ABC, Generic[T]):
    """Abstract base class for model configurations."""

    @classmethod
    @abstractmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """

    @abstractmethod
    def serialize(self) -> Dict[str, Any]:
        """Serialize configuration to dictionary.

        Returns:
            Dict[str, Any]: Serialized configuration object.
        """

    @classmethod
    @abstractmethod
    def deserialize(cls, request_dict: Dict[str, Any]) -> T:
        """Deserialize configuration from request JSON.

        Args:
            request_dict (Dict[str, Any]): Request JSON data.

        Returns:
            T: Deserialized configuration object.
        """



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hume',
 'hume._batch',
 'hume._common',
 'hume._common.config',
 'hume._stream',
 'hume.config']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.26.0,<3.0.0', 'typing-extensions>=4.3.0,<5.0.0']

extras_require = \
{'stream': ['websockets>=10.3,<11.0']}

setup_kwargs = {
    'name': 'hume',
    'version': '0.1.4',
    'description': 'Hume AI Python Client',
    'long_description': '<div align="center">\n  <img src="https://storage.googleapis.com/hume-public-logos/hume/hume-banner.png">\n  <h1>Hume AI Python SDK</h1>\n\n  <p>\n    <strong>Integrate Hume APIs directly into your Python application</strong>\n  </p>\n\n  <br>\n  <div>\n    <a href="https://badge.fury.io/py/hume"><img src="https://badge.fury.io/py/hume.svg" alt="PyPI"></a>\n    <a href="https://pepy.tech/project/hume"><img src="https://pepy.tech/badge/hume" alt="Downloads"></a>\n    <a href="https://github.com/HumeAI/hume-python-sdk/actions/workflows/ci.yml"><img src="https://github.com/HumeAI/hume-python-sdk/actions/workflows/ci.yaml/badge.svg" alt="CI"></a>\n  </div>\n  <br>\n</div>\n\nTo get started, [sign up for a Hume account](https://share.hsforms.com/1lVY-gpw0RTaWCeu7ZTkH-wcjsur)!\n\n> We\'re in private beta right now, but we\'ll be continuously rolling out access to developers and organizations on our waitlist.\n\n## Usage & Documentation\n\nFor usage examples check out the [full documentation](https://humeai.github.io/hume-python-sdk/).\n\n## Other Resources\n\n- [Hume AI Homepage](https://hume.ai)\n- [Platform Documentation](https://help.hume.ai/basics/about-hume-ai)\n- [API Reference](https://docs.hume.ai)\n\n## Support\n\nIf you\'ve found a bug with this SDK please [open an issue](https://github.com/HumeAI/hume-python-sdk/issues/new)!\n',
    'author': 'Hume AI Dev',
    'author_email': 'dev@hume.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/HumeAI/hume-python-sdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)



"""Configuration for the language emotion model."""
from typing import Any, Dict, Optional

from hume._common.config.job_config_base import JobConfigBase
from hume._common.model_type import ModelType


class LanguageConfig(JobConfigBase["LanguageConfig"]):
    """Configuration for the language emotion model."""

    def __init__(
        self,
        *,
        sliding_window: Optional[float] = None,
        identify_speakers: Optional[bool] = None,
    ):
        """Construct a `LanguageConfig`.

        Args:
            sliding_window (Optional[float]): Whether to generate predictions for each word in the text or
                for the entire text in aggregate.
            identify_speakers (Optional[bool]): Whether to return identifiers for speakers over time.
                If true, unique identifiers will be assigned to spoken words to differentiate different speakers.
                If false, all speakers will be tagged with an "unknown" ID.
        """
        self.sliding_window = sliding_window
        self.identify_speakers = identify_speakers

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.LANGUAGE

    def serialize(self) -> Dict[str, Any]:
        """Serialize `LanguageConfig` to dictionary.

        Returns:
            Dict[str, Any]: Serialized `LanguageConfig` object.
        """
        return {
            "sliding_window": self.sliding_window,
            "identify_speakers": self.identify_speakers,
        }

    @classmethod
    def deserialize(cls, request_dict: Dict[str, Any]) -> "LanguageConfig":
        """Deserialize `LanguageConfig` from request JSON.

        Args:
            request_dict (Dict[str, Any]): Request JSON data.

        Returns:
            LanguageConfig: Deserialized `LanguageConfig` object.
        """
        return cls(
            sliding_window=request_dict.get("sliding_window"),
            identify_speakers=request_dict.get("identify_speakers"),
        )



"""Module init."""
from hume._batch.batch_job import BatchJob
from hume._batch.batch_job_result import BatchJobResult
from hume._batch.batch_job_status import BatchJobStatus
from hume._batch.hume_batch_client import HumeBatchClient

__all__ = [
    "BatchJob",
    "BatchJobResult",
    "BatchJobStatus",
    "HumeBatchClient",
]
